# -*- coding: utf-8 -*-
"""Fetch travis logs."""
from __future__ import absolute_import
from __future__ import unicode_literals

from travis_log_fetch.cmdline import main

__version__ = '0.1.0'

main
